@@ -29,6 +29,7 @@ struct xapian_fts_backend
         char * path;
 	long partial,full;
 	bool attachments;
+	bool doexpunge;
 
 	char * guid;
 	char * boxname;
@@ -94,6 +95,7 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 	backend->old_boxname = NULL;
 	backend->attachments = false;
 	backend->db_expunge = NULL;
+	backend->doexpunge = false;
 	verbose = 0;
 	backend->partial = 0;
 	backend->full = 0;
@@ -174,6 +176,8 @@ static int fts_backend_xapian_init(struct fts_backend *_backend, const char **er
 
 static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_deinit");
+
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
 
@@ -196,6 +200,8 @@ static void fts_backend_xapian_deinit(struct fts_backend *_backend)
 
 static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct mailbox *box, uint32_t *last_uid_r)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_get_last_uid");
+
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *)_backend;
 
@@ -232,6 +238,8 @@ static int fts_backend_xapian_get_last_uid(struct fts_backend *_backend, struct
 
 static struct fts_backend_update_context * fts_backend_xapian_update_init(struct fts_backend *_backend)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_update_context");
+
 	struct xapian_fts_backend_update_context *ctx;
 
 	ctx = i_new(struct xapian_fts_backend_update_context, 1);
@@ -241,6 +249,8 @@ static struct fts_backend_update_context * fts_backend_xapian_update_init(struct
 
 static int fts_backend_xapian_update_deinit(struct fts_backend_update_context *_ctx)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_deinit");
+
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend =
@@ -255,6 +265,8 @@ static int fts_backend_xapian_update_deinit(struct fts_backend_update_context *_
 
 static void fts_backend_xapian_update_set_mailbox(struct fts_backend_update_context *_ctx, struct mailbox *box)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_set_mailbox");
+
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend =
@@ -265,6 +277,8 @@ static void fts_backend_xapian_update_set_mailbox(struct fts_backend_update_cont
 
 static void fts_backend_xapian_update_expunge(struct fts_backend_update_context *_ctx, uint32_t uid)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_expunge");
+
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend =
@@ -303,6 +317,8 @@ static void fts_backend_xapian_update_expunge(struct fts_backend_update_context
 
 static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_context *_ctx, const struct fts_backend_build_key *key)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_set_build_key");
+
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
 
@@ -318,6 +334,8 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 		return FALSE;
 	}
 
+	backend->doexpunge = true;
+
 	if((backend->old_guid == NULL) || (strcmp(backend->old_guid,backend->guid)!=0))
         {
 		fts_backend_xapian_oldbox(backend);
@@ -418,6 +436,8 @@ static bool fts_backend_xapian_update_set_build_key(struct fts_backend_update_co
 
 static void fts_backend_xapian_update_unset_build_key(struct fts_backend_update_context *_ctx)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_unset_build_key");
+
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
 
@@ -431,6 +451,8 @@ static void fts_backend_xapian_update_unset_build_key(struct fts_backend_update_
 
 static int fts_backend_xapian_refresh(struct fts_backend * _backend)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_refresh");
+
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *) _backend;
 
@@ -439,7 +461,6 @@ static int fts_backend_xapian_refresh(struct fts_backend * _backend)
         long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
 
 	fts_backend_xapian_release(backend,"refresh");
-	fts_backend_xapian_expunge(backend,"refresh");
 	fts_backend_xapian_commit(backend,"refresh", current_time);
         
 	return 0;
@@ -447,6 +468,8 @@ static int fts_backend_xapian_refresh(struct fts_backend * _backend)
 
 static int fts_backend_xapian_update_build_more(struct fts_backend_update_context *_ctx, const unsigned char *data, size_t size)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_update_build_more");
+
 	struct xapian_fts_backend_update_context *ctx =
 		(struct xapian_fts_backend_update_context *)_ctx;
 	struct xapian_fts_backend *backend =
@@ -487,7 +510,7 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 	{
 		if(verbose>1) i_info("FTS Xapian: Refreshing after %ld ms and %ld updates...", current_time - backend->commit_time, backend->commit_updates);
 		fts_backend_xapian_release(backend,"refreshing");
-		fts_backend_xapian_expunge(backend,"refreshing");
+		//fts_backend_xapian_expunge(backend,"refreshing",XAPIAN_EXPUNGE_SIZE);
 		fts_backend_xapian_commit(backend,"refreshing", current_time);
 	}
     	
@@ -497,6 +520,8 @@ static int fts_backend_xapian_update_build_more(struct fts_backend_update_contex
 
 static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_optimize");
+
 	struct xapian_fts_backend *backend =
 		(struct xapian_fts_backend *) _backend;
 
@@ -506,7 +531,7 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
         gettimeofday(&tp, NULL);
         long current_time = tp.tv_sec * 1000 + tp.tv_usec / 1000;
 	
-	fts_backend_xapian_expunge(backend,"optimize");
+	fts_backend_xapian_expunge(backend,"optimize",0);
         fts_backend_xapian_commit(backend,"optimize",current_time);
 
 	return 0;
@@ -514,6 +539,8 @@ static int fts_backend_xapian_optimize(struct fts_backend *_backend)
 
 static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_rescan");
+
 	struct xapian_fts_backend *backend =
                 (struct xapian_fts_backend *) _backend;
 
@@ -529,6 +556,8 @@ static int fts_backend_xapian_rescan(struct fts_backend *_backend)
 
 static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbox *box, struct mail_search_arg *args, enum fts_lookup_flags flags, struct fts_result *result)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup");
+
 	struct xapian_fts_backend *backend =
                 (struct xapian_fts_backend *) _backend;
 
@@ -600,6 +629,8 @@ static int fts_backend_xapian_lookup(struct fts_backend *_backend, struct mailbo
 
 static int fts_backend_xapian_lookup_multi (struct fts_backend *_backend, struct mailbox *const boxes[], struct mail_search_arg *args, enum fts_lookup_flags flags, struct fts_multi_result *result)
 {
+	if(verbose>0) i_info("FTS Xapian: fts_backend_xapian_lookup_multi");
+
 	struct xapian_fts_backend *backend =
                 (struct xapian_fts_backend *) _backend;
 
